package net.apexteam.apex_flutter_plugin

import android.content.Context
import android.os.Build
import android.provider.Settings
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import java.nio.charset.Charset
import java.security.MessageDigest
import java.security.NoSuchAlgorithmException
import java.util.*

/** ApexFlutterPlugin */
class ApexFlutterPlugin : FlutterPlugin, MethodCallHandler {

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        val channel = MethodChannel(flutterPluginBinding.flutterEngine.dartExecutor, "apex_flutter_plugin")
        channel.setMethodCallHandler(ApexFlutterPlugin());
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "getFingerprint" -> {
                result.success(Build.FINGERPRINT)
            }
            "generateRandomUuid" -> result.success(UUID.randomUUID().toString())
            "getAndroidId" -> result.success(getAndroidId())
            "hashUID" -> result.success(generateHashUID(getAndroidId(), Build.FINGERPRINT))
            else -> {
                result.notImplemented()
            }
        }
    }

    private fun getAndroidId(): String {
        return try {
            Settings.Secure.getString(context?.contentResolver,
                    Settings.Secure.ANDROID_ID)
        } catch (e: Exception) {
            ""
        }
    }

    private fun generateHashUID(vararg items: String): String {
        val originalString = StringBuilder()
        for (element in items) {
            originalString.append(element)
        }
        return try {
            val digest = MessageDigest.getInstance("SHA-256")
            val encodedHash = digest.digest(
                    originalString.toString().toByteArray(Charset.forName("UTF-8")))
            bytesToHex(encodedHash)
        } catch (e: NoSuchAlgorithmException) {
//      e.printStackTrace()
            ""
        }
    }

    private fun bytesToHex(hash: ByteArray): String {
        val hexString = StringBuffer()
        for (i in hash.indices) {
            val hex = Integer.toHexString(0xff and hash[i].toInt())
            if (hex.length == 1) hexString.append('0')
            hexString.append(hex)
        }
        return hexString.toString()
    }

    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    companion object {
      private var context: Context? = null

        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "apex_flutter_plugin")
            channel.setMethodCallHandler(ApexFlutterPlugin())
            context = registrar.activity().applicationContext;
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    }
}
