import 'package:apex_flutter_plugin/api/key_pair.dart';
import 'package:equatable/equatable.dart';

class ApiConfig extends Equatable {
  final bool encrypt;
  final String url;
  final KeyPairs? keyPairs;
  final Duration defaultTimeout;
  final Duration uploadTimeout;
  final String publicVersion;
  final String privateVersion;
  final bool debugMode;
  Map<String, dynamic>? defaultParameters;

  ApiConfig(
    this.url,
    this.publicVersion,
    this.privateVersion, {
    this.keyPairs,
    this.encrypt = false,
    this.defaultParameters,
    this.debugMode = false,
    this.defaultTimeout = const Duration(seconds: 60),
    this.uploadTimeout = const Duration(minutes: 5),
  }) : assert(encrypt == false || (encrypt && keyPairs!.key != null)) {
    if (defaultParameters == null) {
      defaultParameters = {'additional': {}};
    }
  }

  @override
  String toString() {
    return {
      'url': url,
      'encrypt': encrypt,
      'debugMode': debugMode,
      'keyPairs': keyPairs,
      'defaultParameters': defaultParameters,
      'defaultTimeout': defaultTimeout.toString(),
      'uploadTimeout': uploadTimeout.toString(),
    }.toString();
  }

  @override
  List<Object?> get props => [
        url,
        encrypt,
      ];
}
